# -*- coding: utf-8 -*-
# @Time     : 2021/5/10 15:17
# @Author   : ufy
# @Email    : antarm@outlook.com / 549147808@qq.com
# @file     : cutImage.py
# @info     :

from ufy.CV.cutImg import cut_img_energy,cut_img_dir